@@ -1,22 +1,22 @@
 /*
  * libwbxml, the WBXML Library.
- * Copyright (C) 2002-2004 Aymerick Jhanne <aymerick@jehanne.org>
+ * Copyright (C) 2002-2005 Aymerick Jehanne <aymerick@jehanne.org>
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *  
- * GPL v2: http://www.gnu.org/licenses/gpl.txt
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * 
+ * LGPL v2.1: http://www.gnu.org/copyleft/lesser.txt
  * 
  * Contact: libwbxml@aymerick.com
  * Home: http://libwbxml.aymerick.com
@@ -26,7 +26,7 @@
  * @file wbxml_buffers.c
  * @ingroup wbxml_buffers
  *
- * @author Aymerick Jhanne <libwbxml@aymerick.com>
+ * @author Aymerick Jehanne <libwbxml@aymerick.com>
  * @date 02/03/12
  *
  * @brief Generic Buffers Functions
@@ -49,9 +49,10 @@
 struct WBXMLBuffer_s
 {
     WB_UTINY *data;             /**< The data */
-    WB_ULONG len;               /**< Length of data in buffer */
-    WB_ULONG malloced;          /**< Length of buffer */
-    WB_ULONG malloc_block;      /**< Malloc Block Size */
+    WB_ULONG  len;              /**< Length of data in buffer */
+    WB_ULONG  malloced;         /**< Length of buffer */
+    WB_ULONG  malloc_block;     /**< Malloc Block Size */
+    WB_BOOL   is_static;        /**< Is it a static buffer ?  */
 };
 
 
@@ -74,6 +75,7 @@ WBXML_DECLARE(WBXMLBuffer *) wbxml_buffer_create_real(const WB_UTINY *data, WB_U
         return NULL;
         
     buffer->malloc_block = malloc_block;
+    buffer->is_static    = FALSE;
 
     if ((len <= 0) || (data == NULL)) {        
         buffer->malloced = 0;
@@ -101,10 +103,33 @@ WBXML_DECLARE(WBXMLBuffer *) wbxml_buffer_create_real(const WB_UTINY *data, WB_U
 }
 
 
+WBXML_DECLARE(WBXMLBuffer *) wbxml_buffer_sta_create_real(const WB_UTINY *data, WB_ULONG len)
+{
+    WBXMLBuffer *buffer = NULL;
+  
+    buffer = (WBXMLBuffer *) wbxml_malloc(sizeof(WBXMLBuffer));
+    if (buffer == NULL) {
+        return NULL;
+    }
+
+    buffer->malloc_block = 0;
+    buffer->is_static    = TRUE;
+    buffer->data         = (WB_UTINY *) data;
+    buffer->len          = len;
+
+    return buffer;
+}
+
+
 WBXML_DECLARE(void) wbxml_buffer_destroy(WBXMLBuffer *buffer)
 {
     if (buffer != NULL) {
-        wbxml_free(buffer->data);
+        if (!buffer->is_static) {
+            /* Free dynamic data */
+            wbxml_free(buffer->data);
+        }
+
+        /* Free structure */
         wbxml_free(buffer);
     }
 }
@@ -152,7 +177,7 @@ WBXML_DECLARE(WB_BOOL) wbxml_buffer_get_char(WBXMLBuffer *buffer, WB_ULONG pos,
 
 WBXML_DECLARE(void) wbxml_buffer_set_char(WBXMLBuffer *buffer, WB_ULONG pos, WB_UTINY ch)
 {
-    if (pos < buffer->len)
+    if ((buffer != NULL) && !buffer->is_static && (pos < buffer->len))
         buffer->data[pos] = ch;
 }
 
@@ -168,7 +193,7 @@ WBXML_DECLARE(WB_UTINY *) wbxml_buffer_get_cstr(WBXMLBuffer *buffer)
 
 WBXML_DECLARE(WB_BOOL) wbxml_buffer_insert(WBXMLBuffer *to, WBXMLBuffer *buffer, WB_ULONG pos)
 {
-    if ((to != NULL) && (buffer != NULL))
+    if ((to != NULL) && (buffer != NULL) && !to->is_static)
         return insert_data(to, pos, buffer->data, buffer->len);
 
     return FALSE;
@@ -177,7 +202,7 @@ WBXML_DECLARE(WB_BOOL) wbxml_buffer_insert(WBXMLBuffer *to, WBXMLBuffer *buffer,
 
 WBXML_DECLARE(WB_BOOL) wbxml_buffer_insert_cstr(WBXMLBuffer *to, WB_UTINY *str, WB_ULONG pos)
 {
-    if ((to != NULL) && (str != NULL))
+    if ((to != NULL) && (str != NULL) && !to->is_static)
         return insert_data(to, pos, str, WBXML_STRLEN(str));
 
     return FALSE;
@@ -186,7 +211,7 @@ WBXML_DECLARE(WB_BOOL) wbxml_buffer_insert_cstr(WBXMLBuffer *to, WB_UTINY *str,
 
 WBXML_DECLARE(WB_BOOL) wbxml_buffer_append(WBXMLBuffer *dest, WBXMLBuffer *buff)
 {
-    if (dest == NULL)
+    if ((dest == NULL) || dest->is_static)
         return FALSE;
 
     if (buff == NULL)
@@ -198,7 +223,7 @@ WBXML_DECLARE(WB_BOOL) wbxml_buffer_append(WBXMLBuffer *dest, WBXMLBuffer *buff)
 
 WBXML_DECLARE(WB_BOOL) wbxml_buffer_append_data_real(WBXMLBuffer *buffer, const WB_UTINY *data, WB_ULONG len)
 {
-    if (buffer == NULL)
+    if ((buffer == NULL) || buffer->is_static)
         return FALSE;
 
     if ((data == NULL) || (len == 0))
@@ -209,6 +234,10 @@ WBXML_DECLARE(WB_BOOL) wbxml_buffer_append_data_real(WBXMLBuffer *buffer, const
 
 WBXML_DECLARE(WB_BOOL) wbxml_buffer_append_cstr_real(WBXMLBuffer *buffer, const WB_UTINY *data)
 {
+    if ((buffer == NULL) || buffer->is_static) {
+        return FALSE;
+    }
+
     if (data == NULL)
         return TRUE;
 
@@ -220,7 +249,7 @@ WBXML_DECLARE(WB_BOOL) wbxml_buffer_append_char(WBXMLBuffer *buffer, WB_UTINY ch
 {
     WB_UTINY c = ch;
 
-    if ((buffer == NULL) || (ch > UCHAR_MAX))
+    if ((buffer == NULL) || buffer->is_static)
         return FALSE;
 
     return insert_data(buffer, buffer->len, &c, 1);
@@ -229,13 +258,18 @@ WBXML_DECLARE(WB_BOOL) wbxml_buffer_append_char(WBXMLBuffer *buffer, WB_UTINY ch
 
 WBXML_DECLARE(WB_BOOL) wbxml_buffer_append_mb_uint_32(WBXMLBuffer *buffer, WB_ULONG value)
 {
-    /* A uintvar is defined to be up to 32 bits large
+    /**
+     * A uintvar is defined to be up to 32 bits large
      * so it will fit in 5 octets (to handle continuation bits) 
-    */
+     */
     WB_UTINY octets[5];
     WB_LONG i = 0, start = 0;
 
-    /* Handle last byte separately; it has no continuation bit,
+    if ((buffer == NULL) || buffer->is_static)
+        return FALSE;
+
+    /**
+     * Handle last byte separately; it has no continuation bit,
      * and must be encoded even if value is 0. 
      */
     octets[4] = (WB_UTINY) (value & 0x7f);
@@ -253,6 +287,9 @@ WBXML_DECLARE(WB_BOOL) wbxml_buffer_append_mb_uint_32(WBXMLBuffer *buffer, WB_UL
 
 WBXML_DECLARE(void) wbxml_buffer_delete(WBXMLBuffer *buffer, WB_ULONG pos, WB_ULONG len)
 {
+    if ((buffer == NULL) || buffer->is_static)
+        return;
+
     if (pos > buffer->len)
         pos = buffer->len;
         
@@ -274,7 +311,7 @@ WBXML_DECLARE(void) wbxml_buffer_shrink_blanks(WBXMLBuffer *buffer)
     WB_ULONG i = 0, j = 0, end = 0;
     WB_UTINY ch = 0;
     
-    if (buffer == NULL)
+    if ((buffer == NULL) || buffer->is_static)
         return;
         
     end = wbxml_buffer_len(buffer);
@@ -304,10 +341,13 @@ WBXML_DECLARE(void) wbxml_buffer_strip_blanks(WBXMLBuffer *buffer)
     WB_ULONG start = 0, end = 0, len = 0;
     WB_UTINY ch = 0;
 
+    if ((buffer == NULL) || buffer->is_static)
+        return;
+
     /* Remove whitespaces at beginning of buffer... */
     while (wbxml_buffer_get_char(buffer, start, &ch) && 
            isspace(ch) && 
-	       start <= wbxml_buffer_len(buffer))
+           start <= wbxml_buffer_len(buffer))
     {
         start ++;
     }
@@ -350,10 +390,10 @@ WBXML_DECLARE(WB_LONG) wbxml_buffer_compare(WBXMLBuffer *buff1, WBXMLBuffer *buf
 
     if (len == 0) 
     {
-	    if (buff1->len == 0 && buff2->len > 0)
-	        return -1;
-	    if (buff1->len > 0 && buff2->len == 0)
-	        return 1;
+        if (buff1->len == 0 && buff2->len > 0)
+            return -1;
+        if (buff1->len > 0 && buff2->len == 0)
+            return 1;
         return 0;
     }
 
@@ -392,10 +432,10 @@ WBXML_DECLARE(WB_LONG) wbxml_buffer_compare_cstr(WBXMLBuffer *buff, const WB_TIN
 
     if (len == 0) 
     {
-	    if (buff->len == 0 && WBXML_STRLEN(str) > 0)
-	        return -1;
-	    if (buff->len > 0 && WBXML_STRLEN(str) == 0)
-	        return 1;
+        if (buff->len == 0 && WBXML_STRLEN(str) > 0)
+            return -1;
+        if (buff->len > 0 && WBXML_STRLEN(str) == 0)
+            return 1;
         return 0;
     }
 
@@ -458,7 +498,7 @@ WBXML_DECLARE(WB_BOOL) wbxml_buffer_search_char(WBXMLBuffer *to, WB_UTINY ch, WB
 {
     WB_UTINY *p = NULL;
 
-    if ((to == NULL) || (ch > UCHAR_MAX))
+    if (to == NULL)
         return FALSE;
 
     if (pos >= to->len)
@@ -575,7 +615,7 @@ WBXML_DECLARE(void) wbxml_buffer_hex_to_binary(WBXMLBuffer *buffer)
     WB_UTINY *p = NULL;
     WB_ULONG i = 0, len = 0;
 
-    if (buffer == NULL)
+    if ((buffer == NULL) || buffer->is_static)
         return;
 
     p = buffer->data;
@@ -611,7 +651,7 @@ WBXML_DECLARE(WB_BOOL) wbxml_buffer_binary_to_hex(WBXMLBuffer *buffer, WB_BOOL u
     WB_UTINY *hexits = NULL;
     WB_LONG i = 0;
 
-    if (buffer == NULL)
+    if ((buffer == NULL) || buffer->is_static)
         return FALSE;
 
     if (wbxml_buffer_len(buffer) == 0)
@@ -666,7 +706,7 @@ WBXML_DECLARE(void) wbxml_buffer_remove_trailing_zeros(WBXMLBuffer **buffer)
  */
 static WB_BOOL grow_buff(WBXMLBuffer *buffer, WB_ULONG size)
 {
-    if ((buffer == NULL) || (size < 0))
+    if ((buffer == NULL) || buffer->is_static || (size < 0))
         return FALSE;
         
     /* Make room for the invisible terminating NUL */
@@ -697,7 +737,7 @@ static WB_BOOL grow_buff(WBXMLBuffer *buffer, WB_ULONG size)
  */
 static WB_BOOL insert_data(WBXMLBuffer *buffer, WB_ULONG pos, const WB_UTINY *data, WB_ULONG len)
 {
-    if ((buffer == NULL) || (len == 0) || (pos > buffer->len))
+    if ((buffer == NULL) || buffer->is_static || (len == 0) || (pos > buffer->len))
         return FALSE;
 
     if (!grow_buff(buffer, len))
